import os
import boto3

class Client():

    def __init__(self, profile='superadmin', test=False):
        if test:
            self.connection = boto3.client(
                'cloudformation',
                endpoint_url='http://localhost:4581'
            )
        else:
            session = boto3.Session(profile_name=profile)
            self.session = session
            self.connection = session.client(
                'cloudformation'
            )

    def create_stack(self, project, state, tags={}):
        # Start creation process
        aws_tags = [
            {'Key': key, 'Value': value} for key, value in tags.items()
        ] + [
            {
                'Key': 'project',
                'Value': project
            }
        ] if tags else [
            {
                'Key': 'project',
                'Value': project
            }
        ]

        self.connection.create_stack(
            StackName=project,
            TemplateBody=state.to_json(),
            OnFailure='ROLLBACK',
            Tags=aws_tags
        )
        # Wait for Stack creation
        waiter = self.connection.get_waiter('stack_create_complete')
        try:
            waiter.wait(
                StackName=project,
                WaiterConfig={
                    'Delay': 30,
                    'MaxAttempts': 120
                }
            )
        except:
            return False

        # Check Stack Status
        status = self.connection.describe_stacks(
            StackName=project
        )
        results = status.get('Stacks', [])

        # If we have results and they are successful, save our state
        if len(results) > 0:
            if results[0].get('StackStatus', False) == 'CREATE_COMPLETE':
                # return result
                return results[0]
        return False

    def update_stack(self, project, state):
        stack_update = self.connection.update_stack(
            StackName=project,
            TemplateBody=state.to_json(),
        )
        waiter = self.connection.get_waiter('stack_update_complete')
        try:
            waiter.wait(
                StackName=project,
                WaiterConfig={
                    'Delay': 30,
                    'MaxAttempts': 120
                }
            )
        except:
            return False

        # Check Stack Status
        status = self.connection.describe_stacks(
            StackName=project
        )
        results = status.get('Stacks', [])

        # If we have results and they are successful, save our state
        if len(results) > 0:
            if results[0].get('StackStatus', False) == 'UPDATE_COMPLETE':
                return results[0]
        return False

    def describe_stacks(self, project):
        status = self.connection.describe_stacks(
            StackName=project
        )
        return status.get('Stacks', [])
